unit TxtFile;
{$D-,L-,Y-}
{TTxtFile - object-oriented wrapper
 for standard Pascal/Delphi TextFile type.
 Usage :
    var q : TTxtFile;
    q := TTxtFile.Create('MYFILE.TXT');
    q.ReWrite;
    q.WriteLn('Test');
    q.Close;
    q.Free
 }
interface

uses
  SysUtils, Dialogs;

type
  TFileMode = fmClosed..fmOutput;

  TTxtFile = class
  private
     FTextFile   : TextFile;
     FDefaultExt : TFileExt;
     function GetActive : Boolean;
     function GetEof : Boolean;
     function GetEoln : Boolean;
     function GetMode : TFileMode;
     function GetName : TFileName;
     function GetSeekEoln : Boolean;
     procedure SetActive(state : Boolean);
     procedure SetMode(const NewMode : TFileMode);
  public
     procedure Append;
     procedure Assign(Name : TFileName);
     procedure Close; virtual;
     constructor Create(Name : TFileName);
     destructor Destroy; override;
     procedure Flush;
     procedure ReadLn(var S : OpenString);
     procedure ReSet; virtual;{open file for reading}
     procedure ReWrite; virtual;{open file for writing}
     procedure SetTextBuf(var Buf; Size : Word);
     procedure Write(const S : String);
     procedure WriteLn(const S : String);

     {Setting Active to True resets the file if it
      exists and rewrites it otherwise; better set Mode}
     property Active : Boolean read GetActive write SetActive;
     property DefaultExt : TFileExt read FDefaultExt write FDefaultExt;
     property Eof : Boolean read GetEof;
     property Eoln : Boolean read GetEoln;
     property FileName : TFileName read GetName write Assign;
     property Mode : TFileMode read GetMode write SetMode;
     property SeekEoln : Boolean read GetSeekEoln;
  end;{TTxtFile}

implementation

procedure TTxtFile.SetActive(State : Boolean);
begin
   if State then
      if FileExists(FileName) then ReSet
      else ReWrite
   else Close
end;{SetActive}

function TTxtFile.GetActive : Boolean;
begin
   Result := (Mode <> fmClosed)
end;{GetActive}

procedure TTxtFile.Assign(Name : TFileName);
begin
   if Mode <> fmClosed then
      raise EInOutError.Create(
         'Cannot perform this operation on open text file');
   if (Length(ExtractFileExt(Name)) = 0) and
      {maybe the user doesn't want an extension}
      (Name[Length(Name)] <> '.') then
      AppendStr(Name, '.' + DefaultExt);
   System.AssignFile(FTextFile, Name)
end;{Assign}

procedure TTxtFile.Close;
begin
   Mode := fmClosed
end;{Close}

constructor TTxtFile.Create(Name : TFileName);
begin
   inherited Create;
   TTextRec(FTextFile).Mode := fmClosed;
   FDefaultExt := 'TXT';
   if Length(Name) > 0 then
      Assign(Name)
end;{Create}

destructor TTxtFile.Destroy;
begin
   {if the user forgot to close the file
   (*but did not forget to destroy it - good boy*)
    then close it for him/her}
   if Mode <> fmClosed then
      Close;
   inherited Destroy
end;{Destroy}

procedure TTxtFile.Append;
begin
   System.Append(FTextFile);
end;{Append}

function TTxtFile.GetMode : TFileMode;
begin
   Result := TTextRec(FTextFile).Mode
end;{GetMode}

procedure TTxtFile.SetMode(const NewMode : TFileMode);
var CurMod : Word;
begin
   CurMod := Mode;
   if NewMode <> CurMod then begin
      if (NewMode = fmClosed) then begin
         if CurMod = fmOutPut then Self.Flush;
         System.CloseFile(FTextFile)
      end
      else if CurMod > fmClosed{file is currently open} then
         raise EInOutError.Create(
            'Cannot perform this operation on open text file')
      else case NewMode of
         fmInput  : System.ReSet(FTextFile);
         fmOutput : System.ReWrite(FTextFile);
      end;{case}
   end{if}
end;{SetMode}

function TTxtFile.GetName : TFileName;
begin
   Result := StrPas(TTextRec(FTextFile).Name)
end;{GetMode}

procedure TTxtFile.ReWrite;
begin
   Mode := fmOutPut
end;{ReWrite}

procedure TTxtFile.ReSet;
begin
   Mode := fmInput
end;{ReSet}

procedure TTxtFile.SetTextBuf(var Buf; Size : Word);
begin
   System.SetTextBuf(FTextFile, Buf, Size)
end;{SetTextBuf}

function TTxtFile.GetEof : Boolean;
begin
   Result := System.Eof(FTextFile)
end;{GetEof}

function TTxtFile.GetEoln : Boolean;
begin
   Result := System.Eoln(FTextFile)
end;{GetEoln}

function TTxtFile.GetSeekEoln : Boolean;
begin
   Result := System.SeekEoln(FTextFile)
end;{GetSeekEoln}

procedure TTxtFile.ReadLn(var S : OpenString);
begin
   System.Read(FTextFile, S);
   if System.Eoln(FTextFile) then System.ReadLn(FTextFile);
end;{ReadLn}

procedure TTxtFile.WriteLn(const S : String);
begin
   System.WriteLn(FTextFile, S)
end;{WriteLn}

procedure TTxtFile.Write(const S : String);
begin
   System.Write(FTextFile, S)
end;{WriteLn}

procedure TTxtFile.Flush;
begin
   System.Flush(FTextFile)
end;{Flush}

end.{TTxtFile}
